@@ -10,27 +10,33 @@ from rasa.cli.arguments.default_arguments import (
 )
 from rasa.shared.constants import DEFAULT_CONFIG_PATH, DEFAULT_DATA_PATH
 
+USE_LATEST_MODEL_FOR_FINE_TUNING = True
 
-def set_train_arguments(parser: argparse.ArgumentParser):
+
+def set_train_arguments(parser: argparse.ArgumentParser) -> None:
+    """Specifies CLI arguments for `rasa train`."""
     add_data_param(parser)
     add_config_param(parser)
     add_domain_param(parser)
     add_out_param(parser, help_text="Directory where your models should be stored.")
 
+    add_dry_run_param(parser)
     add_augmentation_param(parser)
     add_debug_plots_param(parser)
 
-    add_num_threads_param(parser)
+    _add_num_threads_param(parser)
 
-    add_model_name_param(parser)
+    _add_model_name_param(parser)
     add_persist_nlu_data_param(parser)
     add_force_param(parser)
+    add_finetune_params(parser)
 
 
-def set_train_core_arguments(parser: argparse.ArgumentParser):
+def set_train_core_arguments(parser: argparse.ArgumentParser) -> None:
+    """Specifies CLI arguments for `rasa train core`."""
     add_stories_param(parser)
     add_domain_param(parser)
-    add_core_config_param(parser)
+    _add_core_config_param(parser)
     add_out_param(parser, help_text="Directory where your models should be stored.")
 
     add_augmentation_param(parser)
@@ -38,26 +44,32 @@ def set_train_core_arguments(parser: argparse.ArgumentParser):
 
     add_force_param(parser)
 
-    add_model_name_param(parser)
+    _add_model_name_param(parser)
 
     compare_arguments = parser.add_argument_group("Comparison Arguments")
-    add_compare_params(compare_arguments)
+    _add_compare_params(compare_arguments)
+    add_finetune_params(parser)
 
 
-def set_train_nlu_arguments(parser: argparse.ArgumentParser):
+def set_train_nlu_arguments(parser: argparse.ArgumentParser) -> None:
+    """Specifies CLI arguments for `rasa train nlu`."""
     add_config_param(parser)
     add_domain_param(parser, default=None)
     add_out_param(parser, help_text="Directory where your models should be stored.")
 
     add_nlu_data_param(parser, help_text="File or folder containing your NLU data.")
 
-    add_num_threads_param(parser)
+    _add_num_threads_param(parser)
 
-    add_model_name_param(parser)
+    _add_model_name_param(parser)
     add_persist_nlu_data_param(parser)
+    add_finetune_params(parser)
 
 
-def add_force_param(parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]):
+def add_force_param(
+    parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
+) -> None:
+    """Specifies if the model should be trained from scratch."""
     parser.add_argument(
         "--force",
         action="store_true",
@@ -65,7 +77,10 @@ def add_force_param(parser: Union[argparse.ArgumentParser, argparse._ActionsCont
     )
 
 
-def add_data_param(parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]):
+def add_data_param(
+    parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
+) -> None:
+    """Specifies path to training data."""
     parser.add_argument(
         "--data",
         default=[DEFAULT_DATA_PATH],
@@ -74,7 +89,7 @@ def add_data_param(parser: Union[argparse.ArgumentParser, argparse._ActionsConta
     )
 
 
-def add_core_config_param(parser: argparse.ArgumentParser):
+def _add_core_config_param(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "-c",
         "--config",
@@ -86,9 +101,9 @@ def add_core_config_param(parser: argparse.ArgumentParser):
     )
 
 
-def add_compare_params(
+def _add_compare_params(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
-):
+) -> None:
     parser.add_argument(
         "--percentages",
         nargs="*",
@@ -101,9 +116,38 @@ def add_compare_params(
     )
 
 
+def add_dry_run_param(
+    parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
+) -> None:
+    """Adds `--dry-run` argument to a specified `parser`.
+
+    Args:
+        parser: An instance of `ArgumentParser` or `_ActionsContainer`.
+    """
+    parser.add_argument(
+        "--dry-run",
+        default=False,
+        action="store_true",
+        help="If enabled, no actual training will be performed. Instead, "
+        "it will be determined whether a model should be re-trained "
+        "and this information will be printed as the output. The return "
+        "code is a 4-bit bitmask that can also be used to determine what exactly needs "
+        "to be retrained:\n"
+        "- 1 means Core needs to be retrained\n"
+        "- 2 means NLU needs to be retrained\n"
+        "- 4 means responses in the domain should be updated\n"
+        "- 8 means the training was forced (--force argument is specified)",
+    )
+
+
 def add_augmentation_param(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
-):
+) -> None:
+    """Sets the augmentation factor for the Core training.
+
+    Args:
+        parser: An instance of `ArgumentParser` or `_ActionsContainer`.
+    """
     parser.add_argument(
         "--augmentation",
         type=int,
@@ -114,7 +158,8 @@ def add_augmentation_param(
 
 def add_debug_plots_param(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
-):
+) -> None:
+    """Specifies if conversation flow should be visualized."""
     parser.add_argument(
         "--debug-plots",
         default=False,
@@ -125,9 +170,9 @@ def add_debug_plots_param(
     )
 
 
-def add_num_threads_param(
+def _add_num_threads_param(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
-):
+) -> None:
     parser.add_argument(
         "--num-threads",
         type=int,
@@ -136,7 +181,7 @@ def add_num_threads_param(
     )
 
 
-def add_model_name_param(parser: argparse.ArgumentParser):
+def _add_model_name_param(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "--fixed-model-name",
         type=str,
@@ -147,9 +192,35 @@ def add_model_name_param(parser: argparse.ArgumentParser):
 
 def add_persist_nlu_data_param(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
-):
+) -> None:
+    """Adds parameters for persisting the NLU training data with the model."""
     parser.add_argument(
         "--persist-nlu-data",
         action="store_true",
-        help="Persist the nlu training data in the saved model.",
+        help="Persist the NLU training data in the saved model.",
+    )
+
+
+def add_finetune_params(
+    parser: Union[argparse.ArgumentParser, argparse._ActionsContainer]
+) -> None:
+    """Adds parameters for model finetuning."""
+    parser.add_argument(
+        "--finetune",
+        nargs="?",
+        # If the user doesn't specify `--finetune` at all
+        default=None,
+        # If the user only specifies `--finetune` without an additional path
+        const=USE_LATEST_MODEL_FOR_FINE_TUNING,
+        help="Fine-tune a previously trained model. If no model path is provided, Rasa "
+        "Open Source will try to finetune the latest trained model from the "
+        "model directory specified via '--out'.",
+    )
+
+    parser.add_argument(
+        "--epoch-fraction",
+        type=float,
+        default=1.0,
+        help="Fraction of epochs which are currently specified in the model "
+        "configuration which should be used when finetuning a model.",
     )
