@@ -16,7 +16,6 @@ from rasa.shared.constants import (
     DEFAULT_DOMAIN_PATH,
     DEFAULT_DATA_PATH,
 )
-
 import rasa.utils.common
 
 
@@ -56,13 +55,23 @@ def add_subparser(
     )
     train_nlu_parser.set_defaults(func=train_nlu)
 
-    train_parser.set_defaults(func=train)
+    train_parser.set_defaults(func=lambda args: train(args, can_exit=True))
 
     train_arguments.set_train_core_arguments(train_core_parser)
     train_arguments.set_train_nlu_arguments(train_nlu_parser)
 
 
-def train(args: argparse.Namespace) -> Optional[Text]:
+def train(args: argparse.Namespace, can_exit: bool = False) -> Optional[Text]:
+    """Trains a model.
+
+    Args:
+        args: Namespace arguments.
+        can_exit: If `True`, the operation can send `sys.exit` in the case
+            training was not successful.
+
+    Returns:
+        Path to a trained model or `None` if training was not successful.
+    """
     import rasa
 
     domain = rasa.cli.utils.get_validated_path(
@@ -78,22 +87,46 @@ def train(args: argparse.Namespace) -> Optional[Text]:
         for f in args.data
     ]
 
-    return rasa.train(
+    training_result = rasa.train(
         domain=domain,
         config=config,
         training_files=training_files,
         output=args.out,
+        dry_run=args.dry_run,
         force_training=args.force,
         fixed_model_name=args.fixed_model_name,
         persist_nlu_training_data=args.persist_nlu_data,
         core_additional_arguments=extract_core_additional_arguments(args),
         nlu_additional_arguments=extract_nlu_additional_arguments(args),
+        model_to_finetune=_model_for_finetuning(args),
+        finetuning_epoch_fraction=args.epoch_fraction,
     )
+    if training_result.code != 0 and can_exit:
+        sys.exit(training_result.code)
+
+    return training_result.model
+
+
+def _model_for_finetuning(args: argparse.Namespace) -> Optional[Text]:
+    if args.finetune == train_arguments.USE_LATEST_MODEL_FOR_FINE_TUNING:
+        # We use this constant to signal that the user specified `--finetune` but
+        # didn't provide a path to a model. In this case we try to load the latest
+        # model from the output directory (that's usually models/).
+        return args.out
 
 
 def train_core(
     args: argparse.Namespace, train_path: Optional[Text] = None
 ) -> Optional[Text]:
+    """Trains a Rasa Core model only.
+
+    Args:
+        args: Command-line arguments to configure training.
+        train_path: Path where trained model but not unzipped model should be stored.
+
+    Returns:
+        Path to a trained model or `None` if training was not successful.
+    """
     from rasa.train import train_core
 
     output = train_path or args.out
@@ -122,6 +155,8 @@ def train_core(
             train_path=train_path,
             fixed_model_name=args.fixed_model_name,
             additional_arguments=additional_arguments,
+            model_to_finetune=_model_for_finetuning(args),
+            finetuning_epoch_fraction=args.epoch_fraction,
         )
     else:
         from rasa.core.train import do_compare_training
@@ -134,6 +169,15 @@ def train_core(
 def train_nlu(
     args: argparse.Namespace, train_path: Optional[Text] = None
 ) -> Optional[Text]:
+    """Trains an NLU model.
+
+    Args:
+        args: Namespace arguments.
+        train_path: Directory where models should be stored.
+
+    Returns:
+        Path to a trained model or `None` if training was not successful.
+    """
     from rasa.train import train_nlu
 
     output = train_path or args.out
@@ -157,6 +201,8 @@ def train_nlu(
         persist_nlu_training_data=args.persist_nlu_data,
         additional_arguments=extract_nlu_additional_arguments(args),
         domain=args.domain,
+        model_to_finetune=_model_for_finetuning(args),
+        finetuning_epoch_fraction=args.epoch_fraction,
     )
 
 
