@@ -1,22 +1,24 @@
 # this builtin is needed so we can overwrite in test
+import asyncio
 import json
 import logging
-import asyncio
 import os
-from typing import Text, Optional, Dict, List
 
 import aiohttp
 import questionary
 from aiohttp import ClientTimeout
 from prompt_toolkit.styles import Style
+from typing import Any
+from typing import Text, Optional, Dict, List
 
+import rasa.shared.utils.cli
+import rasa.shared.utils.io
 from rasa.cli import utils as cli_utils
 from rasa.core import utils
-from rasa.core.channels.channel import RestInput
+from rasa.core.channels.rest import RestInput
 from rasa.core.constants import DEFAULT_SERVER_URL
-from rasa.core.interpreter import INTENT_MESSAGE_PREFIX
-from rasa.utils.io import DEFAULT_ENCODING
-from typing import Any, Coroutine
+from rasa.shared.constants import INTENT_MESSAGE_PREFIX
+from rasa.shared.utils.io import DEFAULT_ENCODING
 
 logger = logging.getLogger(__name__)
 
@@ -24,53 +26,81 @@ STREAM_READING_TIMEOUT_ENV = "RASA_SHELL_STREAM_READING_TIMEOUT_IN_SECONDS"
 DEFAULT_STREAM_READING_TIMEOUT_IN_SECONDS = 10
 
 
-def print_bot_output(
-    message: Dict[Text, Any], color=cli_utils.bcolors.OKBLUE
+def print_buttons(
+    message: Dict[Text, Any],
+    is_latest_message: bool = False,
+    color=rasa.shared.utils.io.bcolors.OKBLUE,
 ) -> Optional[questionary.Question]:
-    if ("text" in message) and not ("buttons" in message):
-        cli_utils.print_color(message.get("text"), color=color)
-
-    if "image" in message:
-        cli_utils.print_color("Image: " + message.get("image"), color=color)
-
-    if "attachment" in message:
-        cli_utils.print_color("Attachment: " + message.get("attachment"), color=color)
-
-    if "buttons" in message:
+    if is_latest_message:
         choices = cli_utils.button_choices_from_message_data(
             message, allow_free_text_input=True
         )
-
         question = questionary.select(
             message.get("text"),
             choices,
             style=Style([("qmark", "#6d91d3"), ("", "#6d91d3"), ("answer", "#b373d6")]),
         )
         return question
+    else:
+        rasa.shared.utils.cli.print_color("Buttons:", color=color)
+        for idx, button in enumerate(message.get("buttons")):
+            rasa.shared.utils.cli.print_color(
+                cli_utils.button_to_string(button, idx), color=color
+            )
+
+
+def print_bot_output(
+    message: Dict[Text, Any],
+    is_latest_message: bool = False,
+    color=rasa.shared.utils.io.bcolors.OKBLUE,
+) -> Optional[questionary.Question]:
+    if "buttons" in message:
+        question = print_buttons(message, is_latest_message, color)
+        if question:
+            return question
+
+    if "text" in message:
+        rasa.shared.utils.cli.print_color(message.get("text"), color=color)
+
+    if "image" in message:
+        rasa.shared.utils.cli.print_color("Image: " + message.get("image"), color=color)
+
+    if "attachment" in message:
+        rasa.shared.utils.cli.print_color(
+            "Attachment: " + message.get("attachment"), color=color
+        )
 
     if "elements" in message:
-        cli_utils.print_color("Elements:", color=color)
+        rasa.shared.utils.cli.print_color("Elements:", color=color)
         for idx, element in enumerate(message.get("elements")):
-            cli_utils.print_color(
+            rasa.shared.utils.cli.print_color(
                 cli_utils.element_to_string(element, idx), color=color
             )
 
     if "quick_replies" in message:
-        cli_utils.print_color("Quick Replies:", color=color)
+        rasa.shared.utils.cli.print_color("Quick Replies:", color=color)
         for idx, element in enumerate(message.get("quick_replies")):
-            cli_utils.print_color(cli_utils.button_to_string(element, idx), color=color)
+            rasa.shared.utils.cli.print_color(
+                cli_utils.button_to_string(element, idx), color=color
+            )
 
     if "custom" in message:
-        cli_utils.print_color("Custom json:", color=color)
-        cli_utils.print_color(json.dumps(message.get("custom"), indent=2), color=color)
+        rasa.shared.utils.cli.print_color("Custom json:", color=color)
+        rasa.shared.utils.cli.print_color(
+            json.dumps(message.get("custom"), indent=2), color=color
+        )
 
 
-def get_user_input(button_question: questionary.Question) -> Optional[Text]:
-    if button_question is not None:
-        response = cli_utils.payload_from_button_question(button_question)
+def get_user_input(previous_response: Optional[Dict[str, Any]]) -> Optional[Text]:
+    button_response = None
+    if previous_response is not None:
+        button_response = print_bot_output(previous_response, is_latest_message=True)
+
+    if button_response is not None:
+        response = cli_utils.payload_from_button_question(button_response)
         if response == cli_utils.FREE_TEXT_INPUT_PROMPT:
             # Re-prompt user with a free text input
-            response = get_user_input(None)
+            response = get_user_input({})
     else:
         response = questionary.text(
             "",
@@ -130,16 +160,16 @@ async def record_messages(
 
     exit_text = INTENT_MESSAGE_PREFIX + "stop"
 
-    cli_utils.print_success(
+    rasa.shared.utils.cli.print_success(
         "Bot loaded. Type a message and press enter "
         "(use '{}' to exit): ".format(exit_text)
     )
 
     num_messages = 0
-    button_question = None
+    previous_response = None
     await asyncio.sleep(0.5)  # Wait for server to start
     while not utils.is_limit_reached(num_messages, max_message_limit):
-        text = get_user_input(button_question)
+        text = get_user_input(previous_response)
 
         if text == exit_text or text is None:
             break
@@ -148,14 +178,20 @@ async def record_messages(
             bot_responses = send_message_receive_stream(
                 server_url, auth_token, sender_id, text
             )
+            previous_response = None
             async for response in bot_responses:
-                button_question = print_bot_output(response)
+                if previous_response is not None:
+                    print_bot_output(previous_response)
+                previous_response = response
         else:
             bot_responses = await send_message_receive_block(
                 server_url, auth_token, sender_id, text
             )
+            previous_response = None
             for response in bot_responses:
-                button_question = print_bot_output(response)
+                if previous_response is not None:
+                    print_bot_output(previous_response)
+                previous_response = response
 
         num_messages += 1
         await asyncio.sleep(0)  # Yield event loop for others coroutines
