@@ -3,8 +3,8 @@ from copy import deepcopy
 from sanic import Blueprint, response
 from sanic.request import Request
 from sanic.response import HTTPResponse
-from telegram import (
-    Bot,
+from telebot import TeleBot
+from telebot.types import (
     InlineKeyboardButton,
     Update,
     InlineKeyboardMarkup,
@@ -20,8 +20,8 @@ from rasa.shared.core.constants import USER_INTENT_RESTART
 logger = logging.getLogger(__name__)
 
 
-class TelegramOutput(Bot, OutputChannel):
-    """Output channel for Telegram"""
+class TelegramOutput(TeleBot, OutputChannel):
+    """Output channel for Telegram."""
 
     # skipcq: PYL-W0236
     @classmethod
@@ -61,31 +61,33 @@ class TelegramOutput(Bot, OutputChannel):
         :button_type reply: reply keyboard
         """
         if button_type == "inline":
+            reply_markup = InlineKeyboardMarkup()
             button_list = [
-                [
-                    InlineKeyboardButton(s["title"], callback_data=s["payload"])
-                    for s in buttons
-                ]
+                InlineKeyboardButton(s["title"], callback_data=s["payload"])
+                for s in buttons
             ]
-            reply_markup = InlineKeyboardMarkup(button_list)
+            reply_markup.row(*button_list)
 
         elif button_type == "vertical":
-            button_list = [
-                [InlineKeyboardButton(s["title"], callback_data=s["payload"])]
+            reply_markup = InlineKeyboardMarkup()
+            [
+                reply_markup.row(
+                    InlineKeyboardButton(s["title"], callback_data=s["payload"])
+                )
                 for s in buttons
             ]
-            reply_markup = InlineKeyboardMarkup(button_list)
 
         elif button_type == "reply":
-            button_list = []
-            for bttn in buttons:
-                if isinstance(bttn, list):
-                    button_list.append([KeyboardButton(s["title"]) for s in bttn])
-                else:
-                    button_list.append([KeyboardButton(bttn["title"])])
             reply_markup = ReplyKeyboardMarkup(
-                button_list, resize_keyboard=True, one_time_keyboard=True
+                resize_keyboard=False, one_time_keyboard=True
             )
+            # drop button_type from button_list
+            button_list = [b for b in buttons if b.get("title")]
+            for idx, button in enumerate(buttons):
+                if isinstance(button, list):
+                    reply_markup.add(KeyboardButton(s["title"]) for s in button)
+                else:
+                    reply_markup.add(KeyboardButton(button["title"]))
         else:
             logger.error(
                 "Trying to send text with buttons for unknown "
@@ -202,11 +204,12 @@ class TelegramInput(InputChannel):
         async def message(request: Request) -> Any:
             if request.method == "POST":
 
-                if not out_channel.get_me()["username"] == self.verify:
+                request_dict = request.json
+                update = Update.de_json(request_dict)
+                if not out_channel.get_me().username == self.verify:
                     logger.debug("Invalid access token, check it matches Telegram")
                     return response.text("failed")
 
-                update = Update.de_json(request.json, out_channel)
                 if self._is_button(update):
                     msg = update.callback_query.message
                     text = update.callback_query.data
@@ -264,7 +267,8 @@ class TelegramInput(InputChannel):
         return telegram_webhook
 
     def get_output_channel(self) -> TelegramOutput:
+        """Loads the telegram channel."""
         channel = TelegramOutput(self.access_token)
-        channel.setWebhook(self.webhook_url)
+        channel.set_webhook(url=self.webhook_url)
 
         return channel
