@@ -3,16 +3,17 @@ import os
 import typing
 from typing import Any, Dict, List, Optional, Text, Type
 
-from rasa.nlu.constants import ENTITIES
+from rasa.nlu.constants import TOKENS_NAMES
+from rasa.shared.nlu.constants import TEXT, ENTITIES
 from rasa.nlu.config import RasaNLUModelConfig
 from rasa.nlu.utils.mitie_utils import MitieNLP
 from rasa.nlu.tokenizers.tokenizer import Token, Tokenizer
 from rasa.nlu.components import Component
 from rasa.nlu.extractors.extractor import EntityExtractor
 from rasa.nlu.model import Metadata
-from rasa.nlu.training_data import Message, TrainingData
-from rasa.utils.common import raise_warning
-import rasa.utils.train_utils as train_utils
+from rasa.shared.nlu.training_data.training_data import TrainingData
+from rasa.shared.nlu.training_data.message import Message
+import rasa.shared.utils.io
 
 logger = logging.getLogger(__name__)
 
@@ -81,7 +82,7 @@ class MitieEntityExtractor(EntityExtractor):
 
         # filter out pre-trained entity examples
         filtered_entity_examples = self.filter_trainable_entities(
-            training_data.training_examples
+            training_data.nlu_examples
         )
 
         for example in filtered_entity_examples:
@@ -98,15 +99,15 @@ class MitieEntityExtractor(EntityExtractor):
     def _prepare_mitie_sample(training_example: Message) -> Any:
         import mitie
 
-        text = training_example.text
-        tokens = train_utils.tokens_without_cls(training_example)
+        text = training_example.get(TEXT)
+        tokens = training_example.get(TOKENS_NAMES[TEXT])
         sample = mitie.ner_training_instance([t.text for t in tokens])
         for ent in training_example.get(ENTITIES, []):
             try:
                 # if the token is not aligned an exception will be raised
                 start, end = MitieEntityExtractor.find_entity(ent, text, tokens)
             except ValueError as e:
-                raise_warning(
+                rasa.shared.utils.io.raise_warning(
                     f"Failed to use example '{text}' to train MITIE "
                     f"entity extractor. Example will be skipped."
                     f"Error: {e}"
@@ -117,7 +118,7 @@ class MitieEntityExtractor(EntityExtractor):
                 # input - e.g. on overlapping entities
                 sample.add_entity(list(range(start, end)), ent["entity"])
             except Exception as e:
-                raise_warning(
+                rasa.shared.utils.io.raise_warning(
                     f"Failed to add entity example "
                     f"'{str(e)}' of sentence '{str(text)}'. "
                     f"Example will be ignored. Reason: "
@@ -136,9 +137,7 @@ class MitieEntityExtractor(EntityExtractor):
             )
 
         ents = self.extract_entities(
-            message.text,
-            train_utils.tokens_without_cls(message),
-            mitie_feature_extractor,
+            message.get(TEXT), message.get(TOKENS_NAMES[TEXT]), mitie_feature_extractor
         )
         extracted = self.add_extractor_name(ents)
         message.set(ENTITIES, message.get(ENTITIES, []) + extracted, add_to_output=True)
