@@ -658,7 +658,7 @@ def initialize_error_reporting() -> None:
 
 @async_generator.asynccontextmanager
 async def track_model_training(
-    training_data: "TrainingDataImporter", model_type: Text
+    training_data: "TrainingDataImporter", model_type: Text, is_finetuning: bool = False
 ) -> typing.AsyncGenerator[None, None]:
     """Track a model training started.
 
@@ -670,6 +670,7 @@ async def track_model_training(
         training_data: Training data used for the training.
         model_type: Specifies the type of training, should be either "rasa", "core"
             or "nlu".
+        is_finetuning: `True` if the model is trained by finetuning another model.
     """
     if not initialize_telemetry():
         # telemetry reporting is disabled. we won't do any reporting
@@ -693,7 +694,7 @@ async def track_model_training(
             "policies": config.get("policies"),
             "num_intent_examples": len(nlu_data.intent_examples),
             "num_entity_examples": len(nlu_data.entity_examples),
-            "num_actions": len(domain.action_names),
+            "num_actions": len(domain.action_names_or_texts),
             # Old nomenclature from when 'responses' were still called
             # 'templates' in the domain
             "num_templates": len(domain.templates),
@@ -705,6 +706,7 @@ async def track_model_training(
             "num_lookup_tables": len(nlu_data.lookup_tables),
             "num_synonyms": len(nlu_data.entity_synonyms),
             "num_regexes": len(nlu_data.regex_features),
+            "is_finetuning": is_finetuning,
         },
     )
     start = datetime.now()
@@ -870,7 +872,7 @@ def track_project_init(path: Text) -> None:
         path: Location of the project
     """
     _track(
-        TELEMETRY_PROJECT_CREATED_EVENT, {"init_directory": _hash_directory_path(path)},
+        TELEMETRY_PROJECT_CREATED_EVENT, {"init_directory": _hash_directory_path(path)}
     )
 
 
