@@ -2,7 +2,15 @@ import asyncio
 import os
 import tempfile
 from contextlib import ExitStack
-from typing import Text, Optional, List, Union, Dict
+from typing import (
+    Text,
+    NamedTuple,
+    Tuple,
+    Optional,
+    List,
+    Union,
+    Dict,
+)
 
 import rasa.core.interpreter
 from rasa.shared.nlu.interpreter import NaturalLanguageInterpreter
@@ -10,16 +18,17 @@ from rasa.shared.importers.importer import TrainingDataImporter
 from rasa import model, telemetry
 from rasa.model import FingerprintComparisonResult
 from rasa.shared.core.domain import Domain
+import rasa.shared.utils.common
 from rasa.nlu.model import Interpreter
 import rasa.utils.common
+import rasa.shared.utils.common
 from rasa.utils.common import TempDirectoryPath
 
 from rasa.shared.utils.cli import (
     print_success,
     print_warning,
-    print_error,
-    print_color,
 )
+import rasa.shared.exceptions
 import rasa.shared.utils.io
 from rasa.shared.constants import (
     DEFAULT_MODELS_PATH,
@@ -27,30 +36,75 @@ from rasa.shared.constants import (
     DEFAULT_NLU_SUBDIRECTORY_NAME,
 )
 
+from rasa.core.agent import Agent
+
+CODE_CORE_NEEDS_TO_BE_RETRAINED = 0b0001
+CODE_NLU_NEEDS_TO_BE_RETRAINED = 0b0010
+CODE_NLG_NEEDS_TO_BE_RETRAINED = 0b0100
+CODE_FORCED_TRAINING = 0b1000
+
+
+class TrainingResult(NamedTuple):
+    """Holds information about the results of training."""
+
+    model: Optional[Text] = None
+    code: int = 0
+
 
 def train(
     domain: Text,
     config: Text,
     training_files: Union[Text, List[Text]],
     output: Text = DEFAULT_MODELS_PATH,
+    dry_run: bool = False,
     force_training: bool = False,
     fixed_model_name: Optional[Text] = None,
     persist_nlu_training_data: bool = False,
     core_additional_arguments: Optional[Dict] = None,
     nlu_additional_arguments: Optional[Dict] = None,
     loop: Optional[asyncio.AbstractEventLoop] = None,
-) -> Optional[Text]:
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
+) -> TrainingResult:
+    """Runs Rasa Core and NLU training in `async` loop.
+
+    Args:
+        domain: Path to the domain file.
+        config: Path to the config for Core and NLU.
+        training_files: Paths to the training data for Core and NLU.
+        output: Output path.
+        dry_run: If `True` then no training will be done, and the information about
+            whether the training needs to be done will be printed.
+        force_training: If `True` retrain model even if data has not changed.
+        fixed_model_name: Name of model to be stored.
+        persist_nlu_training_data: `True` if the NLU training data should be persisted
+            with the model.
+        core_additional_arguments: Additional training parameters for core training.
+        nlu_additional_arguments: Additional training parameters forwarded to training
+            method of each NLU component.
+        loop: Optional EventLoop for running coroutines.
+        model_to_finetune: Optional path to a model which should be finetuned or
+            a directory in case the latest trained model should be used.
+        finetuning_epoch_fraction: The fraction currently specified training epochs
+            in the model configuration which should be used for finetuning.
+
+    Returns:
+        An instance of `TrainingResult`.
+    """
     return rasa.utils.common.run_in_loop(
         train_async(
             domain=domain,
             config=config,
             training_files=training_files,
-            output_path=output,
+            output=output,
+            dry_run=dry_run,
             force_training=force_training,
             fixed_model_name=fixed_model_name,
             persist_nlu_training_data=persist_nlu_training_data,
             core_additional_arguments=core_additional_arguments,
             nlu_additional_arguments=nlu_additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         ),
         loop,
     )
@@ -60,13 +114,16 @@ async def train_async(
     domain: Union[Domain, Text],
     config: Text,
     training_files: Optional[Union[Text, List[Text]]],
-    output_path: Text = DEFAULT_MODELS_PATH,
+    output: Text = DEFAULT_MODELS_PATH,
+    dry_run: bool = False,
     force_training: bool = False,
     fixed_model_name: Optional[Text] = None,
     persist_nlu_training_data: bool = False,
     core_additional_arguments: Optional[Dict] = None,
     nlu_additional_arguments: Optional[Dict] = None,
-) -> Optional[Text]:
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
+) -> TrainingResult:
     """Trains a Rasa model (Core and NLU).
 
     Args:
@@ -74,50 +131,58 @@ async def train_async(
         config: Path to the config for Core and NLU.
         training_files: Paths to the training data for Core and NLU.
         output_path: Output path.
+        dry_run: If `True` then no training will be done, and the information about
+            whether the training needs to be done will be printed.
         force_training: If `True` retrain model even if data has not changed.
         fixed_model_name: Name of model to be stored.
         persist_nlu_training_data: `True` if the NLU training data should be persisted
-                                   with the model.
+            with the model.
         core_additional_arguments: Additional training parameters for core training.
         nlu_additional_arguments: Additional training parameters forwarded to training
-                                  method of each NLU component.
+            method of each NLU component.
+        model_to_finetune: Optional path to a model which should be finetuned or
+            a directory in case the latest trained model should be used.
+        finetuning_epoch_fraction: The fraction currently specified training epochs
+            in the model configuration which should be used for finetuning.
 
     Returns:
-        Path of the trained model archive.
+        An instance of `TrainingResult`.
     """
-
     file_importer = TrainingDataImporter.load_from_config(
         config, domain, training_files
     )
-    with ExitStack() as stack:
-        train_path = stack.enter_context(TempDirectoryPath(tempfile.mkdtemp()))
-
+    with TempDirectoryPath(tempfile.mkdtemp()) as train_path:
         domain = await file_importer.get_domain()
 
         if domain.is_empty():
-            return await handle_domain_if_not_exists(
-                file_importer, output_path, fixed_model_name
+            nlu_model = await handle_domain_if_not_exists(
+                file_importer, output, fixed_model_name
             )
+            return TrainingResult(model=nlu_model)
 
         return await _train_async_internal(
             file_importer,
             train_path,
-            output_path,
+            output,
+            dry_run,
             force_training,
             fixed_model_name,
             persist_nlu_training_data,
             core_additional_arguments=core_additional_arguments,
             nlu_additional_arguments=nlu_additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
 
 
 async def handle_domain_if_not_exists(
     file_importer: TrainingDataImporter, output_path, fixed_model_name
 ):
+    """Trains only the nlu model and prints a warning about missing domain."""
     nlu_model_only = await _train_nlu_with_validated_data(
         file_importer, output=output_path, fixed_model_name=fixed_model_name
     )
-    print_warning(
+    rasa.shared.utils.cli.print_warning(
         "Core training was skipped because no valid domain file was found. "
         "Only an NLU-model was created. Please specify a valid domain using "
         "the '--domain' argument or check if the provided domain file exists."
@@ -125,76 +190,155 @@ async def handle_domain_if_not_exists(
     return nlu_model_only
 
 
+def dry_run_result(
+    fingerprint_comparison: FingerprintComparisonResult,
+) -> Tuple[int, List[Text]]:
+    """Returns a dry run result.
+
+    Args:
+        fingerprint_comparison: A result of fingerprint comparison operation.
+
+    Returns:
+        A tuple where the first element is the result code and the second
+        is the list of human-readable texts that need to be printed to the end user.
+    """
+    code = 0
+    texts = []
+
+    if fingerprint_comparison.force_training:
+        code = CODE_FORCED_TRAINING
+        texts.append("The training was forced.")
+        return code, texts
+
+    if fingerprint_comparison.core:
+        code += CODE_CORE_NEEDS_TO_BE_RETRAINED
+        texts.append("Core model should be retrained.")
+
+    if fingerprint_comparison.nlu:
+        code += CODE_NLU_NEEDS_TO_BE_RETRAINED
+        texts.append("NLU model should be retrained.")
+
+    if fingerprint_comparison.nlg:
+        code += CODE_NLG_NEEDS_TO_BE_RETRAINED
+        texts.append("Responses in the domain should be updated.")
+
+    if code == 0:
+        texts.append("No training required.")
+
+    return code, texts
+
+
 async def _train_async_internal(
     file_importer: TrainingDataImporter,
     train_path: Text,
     output_path: Text,
+    dry_run: bool,
     force_training: bool,
     fixed_model_name: Optional[Text],
     persist_nlu_training_data: bool,
     core_additional_arguments: Optional[Dict] = None,
     nlu_additional_arguments: Optional[Dict] = None,
-) -> Optional[Text]:
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
+) -> TrainingResult:
     """Trains a Rasa model (Core and NLU). Use only from `train_async`.
 
     Args:
         file_importer: `TrainingDataImporter` which supplies the training data.
         train_path: Directory in which to train the model.
         output_path: Output path.
+        dry_run: If `True` then no training will be done, and the information about
+            whether the training needs to be done will be printed.
         force_training: If `True` retrain model even if data has not changed.
         fixed_model_name: Name of model to be stored.
         persist_nlu_training_data: `True` if the NLU training data should be persisted
-                                   with the model.
+            with the model.
         core_additional_arguments: Additional training parameters for core training.
         nlu_additional_arguments: Additional training parameters forwarded to training
-                                  method of each NLU component.
+            method of each NLU component.
+        model_to_finetune: Optional path to a model which should be finetuned or
+            a directory in case the latest trained model should be used.
+        finetuning_epoch_fraction: The fraction currently specified training epochs
+            in the model configuration which should be used for finetuning.
 
     Returns:
-        Path of the trained model archive.
+        An instance of `TrainingResult`.
     """
-
     stories, nlu_data = await asyncio.gather(
         file_importer.get_stories(), file_importer.get_nlu_data()
     )
 
-    if stories.is_empty() and nlu_data.can_train_nlu_model():
-        print_error(
+    new_fingerprint = await model.model_fingerprint(file_importer)
+    old_model = model.get_latest_model(output_path)
+
+    fingerprint_comparison = model.should_retrain(
+        new_fingerprint, old_model, train_path, force_training=force_training
+    )
+
+    if dry_run:
+        code, texts = dry_run_result(fingerprint_comparison)
+        for text in texts:
+            print_warning(text) if code > 0 else print_success(text)
+        return TrainingResult(code=code)
+
+    if nlu_data.has_e2e_examples():
+        rasa.shared.utils.common.mark_as_experimental_feature("end-to-end training")
+
+    if stories.is_empty() and nlu_data.contains_no_pure_nlu_data():
+        rasa.shared.utils.cli.print_error(
             "No training data given. Please provide stories and NLU data in "
             "order to train a Rasa model using the '--data' argument."
         )
-        return
+        return TrainingResult()
 
     if stories.is_empty():
-        print_warning("No stories present. Just a Rasa NLU model will be trained.")
-        return await _train_nlu_with_validated_data(
+        rasa.shared.utils.cli.print_warning(
+            "No stories present. Just a Rasa NLU model will be trained."
+        )
+        trained_model = await _train_nlu_with_validated_data(
             file_importer,
             output=output_path,
             fixed_model_name=fixed_model_name,
             persist_nlu_training_data=persist_nlu_training_data,
             additional_arguments=nlu_additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
+        return TrainingResult(model=trained_model)
 
-    if nlu_data.can_train_nlu_model():
-        print_warning("No NLU data present. Just a Rasa Core model will be trained.")
-        return await _train_core_with_validated_data(
+    # We will train nlu if there are any nlu example, including from e2e stories.
+    if nlu_data.contains_no_pure_nlu_data() and not nlu_data.has_e2e_examples():
+        rasa.shared.utils.cli.print_warning(
+            "No NLU data present. Just a Rasa Core model will be trained."
+        )
+        trained_model = await _train_core_with_validated_data(
             file_importer,
             output=output_path,
             fixed_model_name=fixed_model_name,
             additional_arguments=core_additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
 
+        return TrainingResult(model=trained_model)
+
     new_fingerprint = await model.model_fingerprint(file_importer)
     old_model = model.get_latest_model(output_path)
 
     if not force_training:
         fingerprint_comparison = model.should_retrain(
-            new_fingerprint, old_model, train_path
+            new_fingerprint,
+            old_model,
+            train_path,
+            has_e2e_examples=nlu_data.has_e2e_examples(),
         )
     else:
         fingerprint_comparison = FingerprintComparisonResult(force_training=True)
 
     if fingerprint_comparison.is_training_required():
-        async with telemetry.track_model_training(file_importer, model_type="rasa"):
+        async with telemetry.track_model_training(
+            file_importer, model_type="rasa",
+        ):
             await _do_training(
                 file_importer,
                 output_path=output_path,
@@ -205,20 +349,22 @@ async def _train_async_internal(
                 core_additional_arguments=core_additional_arguments,
                 nlu_additional_arguments=nlu_additional_arguments,
                 old_model_zip_path=old_model,
+                model_to_finetune=model_to_finetune,
+                finetuning_epoch_fraction=finetuning_epoch_fraction,
             )
-
-        return model.package_model(
+        trained_model = model.package_model(
             fingerprint=new_fingerprint,
             output_directory=output_path,
             train_path=train_path,
             fixed_model_name=fixed_model_name,
         )
+        return TrainingResult(model=trained_model)
 
-    print_success(
+    rasa.shared.utils.cli.print_success(
         "Nothing changed. You can use the old model stored at '{}'."
         "".format(os.path.abspath(old_model))
     )
-    return old_model
+    return TrainingResult(model=old_model)
 
 
 async def _do_training(
@@ -231,6 +377,8 @@ async def _do_training(
     core_additional_arguments: Optional[Dict] = None,
     nlu_additional_arguments: Optional[Dict] = None,
     old_model_zip_path: Optional[Text] = None,
+    model_to_finetune: Optional["Text"] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ):
     if not fingerprint_comparison_result:
         fingerprint_comparison_result = FingerprintComparisonResult()
@@ -244,10 +392,12 @@ async def _do_training(
             fixed_model_name=fixed_model_name,
             persist_nlu_training_data=persist_nlu_training_data,
             additional_arguments=nlu_additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
         interpreter_path = os.path.join(model_path, DEFAULT_NLU_SUBDIRECTORY_NAME)
     else:
-        print_color(
+        rasa.shared.utils.cli.print_color(
             "NLU data/configuration did not change. No need to retrain NLU model.",
             color=rasa.shared.utils.io.bcolors.OKBLUE,
         )
@@ -261,9 +411,11 @@ async def _do_training(
             additional_arguments=core_additional_arguments,
             interpreter=_load_interpreter(interpreter_path)
             or _interpreter_from_previous_model(old_model_zip_path),
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
     elif fingerprint_comparison_result.should_retrain_nlg():
-        print_color(
+        rasa.shared.utils.cli.print_color(
             "Core stories/configuration did not change. "
             "Only the templates section has been changed. A new model with "
             "the updated templates will be created.",
@@ -271,7 +423,7 @@ async def _do_training(
         )
         await model.update_model_with_new_domain(file_importer, train_path)
     else:
-        print_color(
+        rasa.shared.utils.cli.print_color(
             "Core stories/configuration did not change. No need to retrain Core model.",
             color=rasa.shared.utils.io.bcolors.OKBLUE,
         )
@@ -305,6 +457,8 @@ def train_core(
     train_path: Optional[Text] = None,
     fixed_model_name: Optional[Text] = None,
     additional_arguments: Optional[Dict] = None,
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     return rasa.utils.common.run_in_loop(
         train_core_async(
@@ -315,6 +469,8 @@ def train_core(
             train_path=train_path,
             fixed_model_name=fixed_model_name,
             additional_arguments=additional_arguments,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
     )
 
@@ -327,6 +483,8 @@ async def train_core_async(
     train_path: Optional[Text] = None,
     fixed_model_name: Optional[Text] = None,
     additional_arguments: Optional[Dict] = None,
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     """Trains a Core model.
 
@@ -339,27 +497,42 @@ async def train_core_async(
             directory, otherwise in the provided directory.
         fixed_model_name: Name of model to be stored.
         additional_arguments: Additional training parameters.
+        model_to_finetune: Optional path to a model which should be finetuned or
+            a directory in case the latest trained model should be used.
+        finetuning_epoch_fraction: The fraction currently specified training epochs
+            in the model configuration which should be used for finetuning.
 
     Returns:
         If `train_path` is given it returns the path to the model archive,
         otherwise the path to the directory with the trained model files.
 
     """
-
     file_importer = TrainingDataImporter.load_core_importer_from_config(
         config, domain, [stories]
     )
-    domain = await file_importer.get_domain()
+    stories, nlu_data, domain = await asyncio.gather(
+        file_importer.get_stories(),
+        file_importer.get_nlu_data(),
+        file_importer.get_domain(),
+    )
+
+    if nlu_data.has_e2e_examples():
+        rasa.shared.utils.cli.print_error(
+            "Stories file contains e2e stories. Please train using `rasa train` so that"
+            " the NLU model is also trained."
+        )
+        return None
+
     if domain.is_empty():
-        print_error(
+        rasa.shared.utils.cli.print_error(
             "Core training was skipped because no valid domain file was found. "
             "Please specify a valid domain using '--domain' argument or check "
             "if the provided domain file exists."
         )
         return None
 
-    if not await file_importer.get_stories():
-        print_error(
+    if not stories:
+        rasa.shared.utils.cli.print_error(
             "No stories given. Please provide stories in order to "
             "train a Rasa Core model using the '--stories' argument."
         )
@@ -371,6 +544,8 @@ async def train_core_async(
         train_path=train_path,
         fixed_model_name=fixed_model_name,
         additional_arguments=additional_arguments,
+        model_to_finetune=model_to_finetune,
+        finetuning_epoch_fraction=finetuning_epoch_fraction,
     )
 
 
@@ -381,9 +556,10 @@ async def _train_core_with_validated_data(
     fixed_model_name: Optional[Text] = None,
     additional_arguments: Optional[Dict] = None,
     interpreter: Optional[Interpreter] = None,
+    model_to_finetune: Optional["Text"] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     """Train Core with validated training and config data."""
-
     import rasa.core.train
 
     with ExitStack() as stack:
@@ -395,11 +571,35 @@ async def _train_core_with_validated_data(
             _train_path = stack.enter_context(TempDirectoryPath(tempfile.mkdtemp()))
 
         # normal (not compare) training
-        print_color("Training Core model...", color=rasa.shared.utils.io.bcolors.OKBLUE)
+        rasa.shared.utils.cli.print_color(
+            "Training Core model...", color=rasa.shared.utils.io.bcolors.OKBLUE
+        )
         domain, config = await asyncio.gather(
             file_importer.get_domain(), file_importer.get_config()
         )
-        async with telemetry.track_model_training(file_importer, model_type="core"):
+
+        if model_to_finetune:
+            rasa.shared.utils.common.mark_as_experimental_feature(
+                "Incremental Training feature"
+            )
+            model_to_finetune = await _core_model_for_finetuning(
+                model_to_finetune,
+                file_importer=file_importer,
+                finetuning_epoch_fraction=finetuning_epoch_fraction,
+            )
+
+            if not model_to_finetune:
+                rasa.shared.utils.cli.print_error_and_exit(
+                    f"No Core model for finetuning found. Please make sure to either "
+                    f"specify a path to a previous model or to have a finetunable "
+                    f"model within the directory '{output}'."
+                )
+
+        async with telemetry.track_model_training(
+            file_importer,
+            model_type="core",
+            is_finetuning=model_to_finetune is not None,
+        ):
             await rasa.core.train(
                 domain_file=domain,
                 training_resource=file_importer,
@@ -407,8 +607,9 @@ async def _train_core_with_validated_data(
                 policy_config=config,
                 additional_arguments=additional_arguments,
                 interpreter=interpreter,
+                model_to_finetune=model_to_finetune,
             )
-        print_color(
+        rasa.shared.utils.cli.print_color(
             "Core model training completed.", color=rasa.shared.utils.io.bcolors.OKBLUE
         )
 
@@ -426,6 +627,40 @@ async def _train_core_with_validated_data(
         return _train_path
 
 
+async def _core_model_for_finetuning(
+    model_to_finetune: Text,
+    file_importer: TrainingDataImporter,
+    finetuning_epoch_fraction: float = 1.0,
+) -> Optional[Agent]:
+    path_to_archive = model.get_model_for_finetuning(model_to_finetune)
+    if not path_to_archive:
+        return None
+
+    rasa.shared.utils.cli.print_info(
+        f"Loading Core model from {path_to_archive} for finetuning...",
+    )
+
+    with model.unpack_model(path_to_archive) as unpacked:
+        new_fingerprint = await model.model_fingerprint(file_importer)
+        old_fingerprint = model.fingerprint_from_path(unpacked)
+        if not model.can_finetune(old_fingerprint, new_fingerprint, core=True):
+            rasa.shared.utils.cli.print_error_and_exit(
+                "Core model can not be finetuned."
+            )
+
+        config = await file_importer.get_config()
+        agent = Agent.load(
+            unpacked,
+            new_config=config,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
+        )
+        # Agent might be empty if no underlying Core model was found.
+        if agent.domain is not None and agent.policy_ensemble is not None:
+            return agent
+
+        return None
+
+
 def train_nlu(
     config: Text,
     nlu_data: Text,
@@ -435,6 +670,8 @@ def train_nlu(
     persist_nlu_training_data: bool = False,
     additional_arguments: Optional[Dict] = None,
     domain: Optional[Union[Domain, Text]] = None,
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     """Trains an NLU model.
 
@@ -450,14 +687,16 @@ def train_nlu(
         additional_arguments: Additional training parameters which will be passed to
                               the `train` method of each component.
         domain: Path to the optional domain file/Domain object.
-
+        model_to_finetune: Optional path to a model which should be finetuned or
+            a directory in case the latest trained model should be used.
+        finetuning_epoch_fraction: The fraction currently specified training epochs
+            in the model configuration which should be used for finetuning.
 
     Returns:
         If `train_path` is given it returns the path to the model archive,
         otherwise the path to the directory with the trained model files.
 
     """
-
     return rasa.utils.common.run_in_loop(
         _train_nlu_async(
             config,
@@ -468,6 +707,8 @@ def train_nlu(
             persist_nlu_training_data,
             additional_arguments,
             domain=domain,
+            model_to_finetune=model_to_finetune,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
         )
     )
 
@@ -481,9 +722,11 @@ async def _train_nlu_async(
     persist_nlu_training_data: bool = False,
     additional_arguments: Optional[Dict] = None,
     domain: Optional[Union[Domain, Text]] = None,
+    model_to_finetune: Optional[Text] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     if not nlu_data:
-        print_error(
+        rasa.shared.utils.cli.print_error(
             "No NLU data given. Please provide NLU data in order to train "
             "a Rasa NLU model using the '--nlu' argument."
         )
@@ -495,8 +738,8 @@ async def _train_nlu_async(
     )
 
     training_data = await file_importer.get_nlu_data()
-    if training_data.can_train_nlu_model():
-        print_error(
+    if training_data.contains_no_pure_nlu_data():
+        rasa.shared.utils.cli.print_error(
             f"Path '{nlu_data}' doesn't contain valid NLU data in it. "
             f"Please verify the data format. "
             f"The NLU model training will be skipped now."
@@ -510,6 +753,8 @@ async def _train_nlu_async(
         fixed_model_name=fixed_model_name,
         persist_nlu_training_data=persist_nlu_training_data,
         additional_arguments=additional_arguments,
+        model_to_finetune=model_to_finetune,
+        finetuning_epoch_fraction=finetuning_epoch_fraction,
     )
 
 
@@ -520,9 +765,10 @@ async def _train_nlu_with_validated_data(
     fixed_model_name: Optional[Text] = None,
     persist_nlu_training_data: bool = False,
     additional_arguments: Optional[Dict] = None,
+    model_to_finetune: Optional["Text"] = None,
+    finetuning_epoch_fraction: float = 1.0,
 ) -> Optional[Text]:
     """Train NLU with validated training and config data."""
-
     import rasa.nlu.train
 
     if additional_arguments is None:
@@ -536,17 +782,42 @@ async def _train_nlu_with_validated_data(
             # Otherwise, create a temp train path and clean it up on exit.
             _train_path = stack.enter_context(TempDirectoryPath(tempfile.mkdtemp()))
         config = await file_importer.get_config()
-        print_color("Training NLU model...", color=rasa.shared.utils.io.bcolors.OKBLUE)
-        async with telemetry.track_model_training(file_importer, model_type="nlu"):
+        rasa.shared.utils.cli.print_color(
+            "Training NLU model...", color=rasa.shared.utils.io.bcolors.OKBLUE
+        )
+
+        if model_to_finetune:
+            rasa.shared.utils.common.mark_as_experimental_feature(
+                "Incremental Training feature"
+            )
+            model_to_finetune = await _nlu_model_for_finetuning(
+                model_to_finetune,
+                file_importer,
+                finetuning_epoch_fraction,
+                called_from_combined_training=train_path is not None,
+            )
+            if not model_to_finetune:
+                rasa.shared.utils.cli.print_error_and_exit(
+                    f"No NLU model for finetuning found. Please make sure to either "
+                    f"specify a path to a previous model or to have a finetunable "
+                    f"model within the directory '{output}'."
+                )
+
+        async with telemetry.track_model_training(
+            file_importer,
+            model_type="nlu",
+            is_finetuning=model_to_finetune is not None,
+        ):
             await rasa.nlu.train(
                 config,
                 file_importer,
                 _train_path,
                 fixed_model_name="nlu",
                 persist_nlu_training_data=persist_nlu_training_data,
+                model_to_finetune=model_to_finetune,
                 **additional_arguments,
             )
-        print_color(
+        rasa.shared.utils.cli.print_color(
             "NLU model training completed.", color=rasa.shared.utils.io.bcolors.OKBLUE
         )
 
@@ -563,3 +834,42 @@ async def _train_nlu_with_validated_data(
             )
 
         return _train_path
+
+
+async def _nlu_model_for_finetuning(
+    model_to_finetune: Text,
+    file_importer: TrainingDataImporter,
+    finetuning_epoch_fraction: float = 1.0,
+    called_from_combined_training: bool = False,
+) -> Optional[Interpreter]:
+
+    path_to_archive = model.get_model_for_finetuning(model_to_finetune)
+    if not path_to_archive:
+        return None
+
+    rasa.shared.utils.cli.print_info(
+        f"Loading NLU model from {path_to_archive} for finetuning...",
+    )
+    with model.unpack_model(path_to_archive) as unpacked:
+        _, old_nlu = model.get_model_subdirectories(unpacked)
+        new_fingerprint = await model.model_fingerprint(file_importer)
+        old_fingerprint = model.fingerprint_from_path(unpacked)
+        if not model.can_finetune(
+            old_fingerprint,
+            new_fingerprint,
+            nlu=True,
+            core=called_from_combined_training,
+        ):
+            rasa.shared.utils.cli.print_error_and_exit(
+                "NLU model can not be finetuned."
+            )
+
+        config = await file_importer.get_config()
+        model_to_finetune = Interpreter.load(
+            old_nlu,
+            new_config=config,
+            finetuning_epoch_fraction=finetuning_epoch_fraction,
+        )
+        if not model_to_finetune:
+            return None
+    return model_to_finetune
